// Inc2Use.cpp : A simple utility to create MTB "USE" files from
//              .H (header files).
//
//  This utility creates a MTB "USE" file from a header file (usually
//  resource.h) which allows one to maintain the symbol mappings
//  without a lot of manual effort on the part of the programmer
//  or dialog author.
//
//  This is done by naming convention, the naming convention used
//  being that normally accepted as standard Windows and MFC
//  programming practices.

#include <ctype.h>
#include <afx.h>
#include <conio.h>

#include "Inc2Use.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#define SIGNON_VER 1
#define SIGNON_REV 1

/////////////////////////////////////////////////////////////////////////////

void UsageErr(const char* szErrorMessage = NULL,
			  const char* szErrorParam = NULL)
{
	fprintf(stderr,
		"\nComet (R) Include File Conversion Utility Version %d.%02d\n"
		"Copyright (c) Signature Systems Inc. 1997-2005. All rights reserved.\n\n",
		SIGNON_VER, SIGNON_REV);

	if (szErrorMessage != NULL)
	{
		fprintf(stderr, "Inc2Use: error: ");
		fprintf(stderr, szErrorMessage, szErrorParam);
		fprintf(stderr, ".\n\n");
	}

	fprintf(stderr, "Inc2Use usage:\n\n"
		"  Inc2Use [/s] <include.h> [output]\n"
	"\n"
	"    [/s]          - if present will cause the output to be sorted"
	"    <IncludeFile> - identifies the input header file (ie. resource.h).\n"
	"    [UseFile]     - identifies the output MTB USE file.  If not\n"
	"                    supplied, output is written to stdout.\n"
	"\n"
	"Press any key\n");
	_getche();

	exit(1);
}

/////////////////////////////////////////////////////////////////////////////


BOOL CLineFile::ReadLine(CString& str)
{
	UINT nMax = 512;
	for (;;)
	{
		ULONGLONG pos = GetPosition();
		if (!ReadString(str.GetBuffer(nMax), nMax))
			return FALSE;
		str.ReleaseBuffer();
		if (str.GetLength() < (int)nMax-1)
			return TRUE;
		nMax += 128;
		Seek(pos, CFile::begin);
	}
	ASSERT(FALSE);
}

void CLineFile::WriteLine(const CString& str)
{
	ASSERT(str[str.GetLength()-1] == '\n');
	WriteString(str);
}

void CLineFile::SafeOpen(const CString& name, UINT nStyleFlags)
{
	BOOL fSuccess = Open(name, nStyleFlags, 0);
	if (!fSuccess)
		UsageErr("Unable to open file \"%s\"", name);
}


/////////////////////////////////////////////////////////////////////////////

BOOL ConvertLine(CString& strLine)
{
	static char szWhiteSpace1[] = "\t ";
	static char szWhiteSpace2[] = "\t\n ";
	static char szDefine[] = "#define";
	static char szIf[] = "#if";
	static char szIfDef[] = "#ifdef";
	static char szIfNDef[] = "#ifndef";
	static char szEndIf[] = "#endif";
	static int InsideIF = 0;

	char* pszCopy = _strdup(strLine);
	if (pszCopy == NULL)
		{
		AfxThrowMemoryException();
		ASSERT(FALSE);
		}

	char* psz;
	char* pszSymbol;
	char* pszValue;

	psz = strtok(pszCopy, szWhiteSpace1);
	if (psz == NULL)
		goto RetFalse;
	if (!strcmp(psz, szEndIf))
		{
		ASSERT(InsideIF > 0);
		InsideIF--;
		goto RetFalse;
		}
	if (!strcmp(psz, szIf) ||
		!strcmp(psz, szIfNDef) ||
		!strcmp(psz, szIfDef))
		{
		InsideIF++;
		goto RetFalse;
		}

	if (InsideIF)
		goto RetFalse;

	if (strcmp(psz, szDefine))
		goto RetFalse;
	pszSymbol = strtok(NULL, szWhiteSpace1);
	if (pszSymbol == NULL)
		goto RetFalse;
	pszValue = strtok(NULL, szWhiteSpace2);
	if (pszValue == NULL)
		goto RetFalse;

	// Convert '_' to '.'
	while (psz = strchr(pszSymbol, '_'))
		*psz = '.';

	//BLOCK: format output line
	{
	CString strNewName = "SET ";
	strNewName += pszSymbol;
	strNewName += " = ";
	CString strPad;
	if (strNewName.GetLength() < 30)
		strPad = CString(' ', 30-strNewName.GetLength());

	strLine = strNewName + strPad + pszValue + "\n";
	}

	ASSERT(pszCopy != NULL);
	free(pszCopy);
	return TRUE;

RetFalse:
	ASSERT(pszCopy != NULL);
	free(pszCopy);
	return FALSE;
}

/////////////////////////////////////////////////////////////////////////////
// 
// SortableStringArray.cpp
// 

int CSortableStringArray::Compare(const CString * pstr1, const CString * pstr2)
{
	ASSERT(pstr1);
	ASSERT(pstr2);
	return pstr1->Compare(*pstr2);
}

void CSortableStringArray::Sort(STRINGCOMPAREFN pfnCompare /*= CSortedStringArray::Compare */)
{
	CString * prgstr = GetData();
	qsort(prgstr,GetSize(),sizeof(CString),(GENERICCOMPAREFN)pfnCompare);
}

int main(int argc, char** argv)
{
	// must only have 1-2 parms left on command line.
	if (argc < 2)
		{
		UsageErr(NULL, NULL);
		ASSERT(FALSE);
		return 1;
		}

	int FirstArg;
	bool bDoSort;
	CString Arg1 = argv[1];
	Arg1.MakeUpper();
	if (Arg1 == "/S" || Arg1 == "-S")
		{
		argc--;

		// must only have 1-2 parms left on command line.
		if (argc < 2)
			{
			UsageErr(NULL, NULL);
			ASSERT(FALSE);
			return 1;
			}

		bDoSort = true;
		FirstArg = 2;
		}
	else
		{
		bDoSort = false;
		FirstArg = 1;
		}

	// open input file.
	CLineFile fileIn;
	fileIn.SafeOpen(argv[FirstArg], CLineFile::modeRead);

	// open/hook up output file.
	CLineFile fileOut;
	if (argc == 3)
		fileOut.SafeOpen(argv[FirstArg+1], CLineFile::modeWrite | CLineFile::modeCreate);
	else
		fileOut.m_pStream = stdout;

	// process the file.
	CSortableStringArray strLineData;
	CString strLine;
	while (fileIn.ReadLine(strLine))
		{
			if (ConvertLine(strLine))
				strLineData.Add(strLine);
		}

	if (bDoSort)
		strLineData.Sort();

	for (INT_PTR Index = 0 ; Index < strLineData.GetCount() ; Index++)
		{
			fileOut.WriteLine(strLineData[Index]);
		}

	// close files.
	fileIn.Close();
	fileOut.Close();

	return 0;
}

/////////////////////////////////////////////////////////////////////////////
